@@ -1,6 +1,5 @@
 use crate::Endianness;
 use std::cmp::min;
-use std::iter::{once, repeat};
 use std::marker::PhantomData;
 use std::mem::size_of;
 
@@ -29,21 +28,15 @@ impl<'a, E: Endianness> WriteBuffer<'a, E> {
     pub fn push_non_fit_bits<I>(&mut self, bits: I, count: usize)
     where
         I: ExactSizeIterator,
-        I: DoubleEndedIterator<Item = u8>,
+        I: DoubleEndedIterator<Item = (usize, u8)>,
     {
-        let full_bytes = min(bits.len() - 1, count / 8);
-
-        let counts = repeat(8)
-            .take(full_bytes)
-            .chain(once(count - full_bytes * 8));
-        if E::is_le() {
-            bits.zip(counts)
-                .for_each(|(chunk, count)| self.push_bits(chunk as usize, count))
-        } else {
-            bits.take(count / 8 + 1)
-                .rev()
-                .zip(counts)
-                .for_each(|(chunk, count)| self.push_bits(chunk as usize, count))
+        let mut remaining = count;
+        for (chunk, chunk_size) in bits {
+            if remaining > 0 {
+                let bits = min(remaining, chunk_size as usize);
+                self.push_bits(chunk, bits);
+                remaining -= bits
+            }
         }
     }
 
@@ -52,12 +45,14 @@ impl<'a, E: Endianness> WriteBuffer<'a, E> {
         if count == 0 {
             return;
         }
-        debug_assert!(count < USIZE_BITS - 8);
 
         // ensure there are no stray bits
         let bits = bits & (usize::MAX >> (USIZE_BITS - count));
 
         let bit_offset = self.bit_len & 7;
+
+        debug_assert!(count <= USIZE_BITS - bit_offset);
+
         let last_written_byte = if bit_offset > 0 {
             self.bytes.pop().unwrap_or(0)
         } else {
@@ -99,4 +94,20 @@ impl<'a, E: Endianness> WriteBuffer<'a, E> {
         self.bytes.extend_from_slice(slice);
         self.bit_len += slice.len() * 8
     }
+
+    pub fn push_bool(&mut self, val: bool) {
+        let val = val as u8;
+        let bit_offset = self.bit_len() % 8;
+        let shift = if E::is_le() {
+            bit_offset
+        } else {
+            7 - bit_offset
+        };
+        if bit_offset == 0 {
+            self.bytes.push(val << shift);
+        } else {
+            *self.bytes.last_mut().unwrap() |= val << shift;
+        }
+        self.bit_len += 1;
+    }
 }
